@@ -93,120 +93,18 @@ static char* prompt()
     char* buf = readline(SFD(nextout).fdbuf.mBuf);
 
     (void)vector_pop_back(gStackFrames);
+
     stack_end_stack();
 
     return buf;
 }
 
-void xyzsh_shell(char* cmdline, char* source_name, sObject* nextin, sObject* nextout, BOOL add_history_, fXyzshJobDone xyzsh_job_done_, char** argv, int argc)
+BOOL xyzsh_run(int* rcode, sObject* block, char* source_name, fXyzshJobDone xyzsh_job_done_, sObject* nextin, sObject* nextout, int argc, char** argv, sObject* current_object)
 {
-    if(nextin == NULL) nextin = gStdin;
-    if(nextout == NULL) nextout = gStdout;
+    string_put(gErrMsg, "");
 
     xyzsh_job_done = xyzsh_job_done_;
 
-    char* histsize_env = getenv("XYZSH_HISTSIZE");
-    int histsize;
-    if(histsize_env) {
-        histsize = atoi(histsize_env);
-        if(histsize < 0) histsize = 1000;
-    }
-    else {
-        histsize = 1000;
-    }
-
-    HISTORY_STATE* history_state = history_get_history_state();
-    int history_num = history_state->length;
-    if(cmdline) {
-        stack_start_stack();
-        int rcode = 0;
-        sObject* block = BLOCK_NEW_STACK();
-        int sline = 1;
-        if(parse(cmdline, "xyzsh", &sline, block, NULL)) {
-            xyzsh_set_signal();
-
-            sObject* fun = FUN_NEW_STACK(NULL);
-            sObject* stackframe = UOBJECT_NEW_GC(8, gxyzshObject, "_stackframe", FALSE);
-            vector_add(gStackFrames, stackframe);
-            //uobject_init(stackframe);
-            SFUN(fun).mLocalObjects = stackframe;
-
-            sObject* argv2 = VECTOR_NEW_GC(16, FALSE);
-            int i;
-            for(i=0; i<argc; i++) {
-                vector_add(argv2, STRING_NEW_GC(argv[i], FALSE));
-            }
-            hash_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
-
-            SFUN(fun).mRunInfo = NULL;
-
-            if(!run(block, nextin, nextout, &rcode, gCurrentObject, fun)) {
-                if(rcode == RCODE_BREAK) {
-                    fprintf(stderr, "invalid break. Not in a loop\n");
-                }
-                else if(rcode == RCODE_RETURN) {
-                    fprintf(stderr, "invalid return. Not in a function\n");
-                }
-                else {
-                    fprintf(stderr, "run time error\n");
-                    fprintf(stderr, "%s", string_c_str(gErrMsg));
-                }
-            }
-            xyzsh_restore_signal_default();
-
-            (void)vector_pop_back(gStackFrames);
-        }
-        else {
-            fprintf(stderr, "parser error\n");
-            fprintf(stderr, "%s", string_c_str(gErrMsg));
-        }
-
-        /// add history ///
-        if(add_history_) {
-            add_history(cmdline);
-            history_num++;
-            if(history_num > histsize) {
-                HIST_ENTRY* history = remove_history(0);
-                free(history);
-            }
-        }
-
-        if(rcode != 0) {
-            fprintf(stderr, "return code is %d\n", rcode);
-        }
-        stack_end_stack();
-    }
-
-    /// wait background job
-    xyzsh_wait_background_job();
-
-    if(add_history_) {
-        /// read histories ///
-        char* histfile = getenv("XYZSH_HISTFILE");
-
-        if(histfile) {
-            write_history(histfile);
-        }
-        else {
-            char* home = getenv("HOME");
-
-            if(home) {
-                char path[PATH_MAX];
-                snprintf(path, PATH_MAX, "%s/.xyzsh/history", home);
-                write_history(path);
-            }
-            else {
-                fprintf(stderr, "HOME evironment path is NULL. exited\n");
-                exit(1);
-            }
-        }
-    }
-}
-
-int xyzsh_run(sObject* block, char* source_name, fXyzshJobDone xyzsh_job_done_, sObject* nextin, sObject* nextout, int argc, char** argv)
-{
-    xyzsh_job_done = xyzsh_job_done_;
-
     stack_start_stack();
     xyzsh_set_signal();
 
@@ -221,36 +119,30 @@ int xyzsh_run(sObject* block, char* source_name, fXyzshJobDone xyzsh_job_done_,
     for(i=0; i<argc; i++) {
         vector_add(argv2, STRING_NEW_GC(argv[i], FALSE));
     }
-    hash_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
+    uobject_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
 
     SFUN(fun).mRunInfo = NULL;
 
-    int rcode;
-    if(!run(block, nextin, nextout, &rcode, gCurrentObject, fun)) {
-        if(rcode == RCODE_BREAK) {
-            fprintf(stderr, "invalid break. Not in a loop\n");
-        }
-        else if(rcode == RCODE_RETURN) {
-            fprintf(stderr, "invalid return. Not in a function\n");
-        }
-        else {
-            fprintf(stderr, "run time error\n");
-            fprintf(stderr, "%s", string_c_str(gErrMsg));
-        }
+    if(!run(block, nextin, nextout, rcode, current_object, fun)) {
+        xyzsh_restore_signal_default();
+        (void)vector_pop_back(gStackFrames);
+
+        stack_end_stack();
+
+        /// wait background job
+        xyzsh_wait_background_job();
+
+        return FALSE;
     }
     xyzsh_restore_signal_default();
 
     (void)vector_pop_back(gStackFrames);
-
-    if(rcode != 0) {
-        fprintf(stderr, "return code is %d\n", rcode);
-    }
     stack_end_stack();
 
     /// wait background job
     xyzsh_wait_background_job();
 
-    return rcode;
+    return TRUE;
 }
 
 static void readline_insert_text(char* cmdline, int cursor_point)
@@ -346,7 +238,7 @@ void readline_write_history()
     }
 }
 
-void xyzsh_readline_interface_onetime(char* cmdline, int cursor_point, char* source_name, char** argv, int argc, fXyzshJobDone xyzsh_job_done_)
+BOOL xyzsh_readline_interface_onetime(int* rcode, char* cmdline, int cursor_point, char* source_name, char** argv, int argc, fXyzshJobDone xyzsh_job_done_)
 {
     /// start interactive shell ///
     mreset_tty();
@@ -377,7 +269,6 @@ void xyzsh_readline_interface_onetime(char* cmdline, int cursor_point, char* sou
     }
     else if(*buf) {
         stack_start_stack();
-        int rcode = 0;
         sObject* block = BLOCK_NEW_STACK();
         int sline = 1;
         if(parse(buf, "xyzsh", &sline, block, NULL)) {
@@ -394,37 +285,75 @@ void xyzsh_readline_interface_onetime(char* cmdline, int cursor_point, char* sou
             for(i=0; i<argc; i++) {
                 vector_add(argv2, STRING_NEW_GC(argv[i], FALSE));
             }
-            hash_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
+            uobject_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
 
             SFUN(fun).mRunInfo = NULL;
 
-            if(!run(block, gStdin, gStdout, &rcode, gCurrentObject, fun)) {
-                readline_signal();
-                if(rcode == RCODE_BREAK) {
+            if(!run(block, gStdin, gStdout, rcode, gCurrentObject, fun)) {
+                xyzsh_restore_signal_default();
+                
+                if(*rcode == RCODE_BREAK) {
                     fprintf(stderr, "invalid break. Not in a loop\n");
                 }
-                else if(rcode == RCODE_RETURN) {
+                else if(*rcode == RCODE_RETURN) {
                     fprintf(stderr, "invalid return. Not in a function\n");
                 }
-/*
-                else if(rcode == RCODE_EXIT) {
-                    stack_end_stack();
-                    (void)vector_pop_back(gStackFrames);
-                    break;
+                else if(*rcode == RCODE_EXIT) {
                 }
-*/
                 else {
                     fprintf(stderr, "run time error\n");
                     fprintf(stderr, "%s", string_c_str(gErrMsg));
                 }
-            }
-            readline_signal();
 
+                /// add history ///
+                add_history(buf);
+                history_num++;
+                if(history_num > gHistSize) {
+                    HIST_ENTRY* history = remove_history(0);
+                    free(history);
+                }
+
+                if(*rcode != 0) {
+                    fprintf(stderr, "return code is %d\n", *rcode);
+                }
+                (void)vector_pop_back(gStackFrames);
+                stack_end_stack();
+
+                /// wait background job
+                xyzsh_wait_background_job();
+
+                free(buf);
+
+                return FALSE;
+            }
             (void)vector_pop_back(gStackFrames);
+            readline_signal();
         }
         else {
+            xyzsh_restore_signal_default();
+
             fprintf(stderr, "parser error\n");
             fprintf(stderr, "%s", string_c_str(gErrMsg));
+
+            /// add history ///
+            add_history(buf);
+            history_num++;
+            if(history_num > gHistSize) {
+                HIST_ENTRY* history = remove_history(0);
+                free(history);
+            }
+
+            if(*rcode != 0) {
+                fprintf(stderr, "return code is %d\n", *rcode);
+            }
+            stack_end_stack();
+
+            /// wait background job
+            xyzsh_wait_background_job();
+
+            free(buf);
+
+            return FALSE;
         }
 
         /// add history ///
@@ -435,8 +364,8 @@ void xyzsh_readline_interface_onetime(char* cmdline, int cursor_point, char* sou
             free(history);
         }
 
-        if(rcode != 0) {
-            fprintf(stderr, "return code is %d\n", rcode);
+        if(*rcode != 0) {
+            fprintf(stderr, "return code is %d\n", *rcode);
         }
         stack_end_stack();
     }
@@ -445,6 +374,8 @@ void xyzsh_readline_interface_onetime(char* cmdline, int cursor_point, char* sou
     xyzsh_wait_background_job();
 
     free(buf);
+
+    return TRUE;
 }
 
 void xyzsh_readline_interface(char* cmdline, int cursor_point, char** argv, int argc)
@@ -488,7 +419,7 @@ void xyzsh_readline_interface(char* cmdline, int cursor_point, char** argv, int
 
         /// run ///
         if(buf == NULL) {
-            if(vector_size(gJobs) > 0) {
+            if(vector_count(gJobs) > 0) {
                 fprintf(stderr,"\njobs exist\n");
             }
             else {
@@ -514,7 +445,7 @@ void xyzsh_readline_interface(char* cmdline, int cursor_point, char** argv, int
                 for(i=0; i<argc; i++) {
                     vector_add(argv2, STRING_NEW_GC(argv[i], FALSE));
                 }
-                hash_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
+                uobject_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
 
                 SFUN(fun).mRunInfo = NULL;
 
@@ -528,7 +459,6 @@ void xyzsh_readline_interface(char* cmdline, int cursor_point, char** argv, int
                     }
                     else if(rcode == RCODE_EXIT) {
                         stack_end_stack();
-                        (void)vector_pop_back(gStackFrames);
                         break;
                     }
                     else {
@@ -536,9 +466,8 @@ void xyzsh_readline_interface(char* cmdline, int cursor_point, char** argv, int
                         fprintf(stderr, "%s", string_c_str(gErrMsg));
                     }
                 }
-                readline_signal();
-
                 (void)vector_pop_back(gStackFrames);
+                readline_signal();
             }
             else {
                 fprintf(stderr, "parser error\n");
@@ -566,29 +495,34 @@ void xyzsh_readline_interface(char* cmdline, int cursor_point, char** argv, int
     }
 }
 
-void xyzsh_load_file(char* fname, char** argv, int argc)
+BOOL xyzsh_load_file(char* fname, char** argv, int argc, sObject* current_object)
 {
     xyzsh_set_signal_optc();
     sRunInfo runinfo;
     memset(&runinfo, 0, sizeof(sRunInfo));
     runinfo.mSName = "xyzsh";
-    runinfo.mCurrentObject = gRootObject;
+    runinfo.mCurrentObject = current_object;
     if(!load_file(fname, gStdin, gStdout, &runinfo, argv, argc))
     {
         if(runinfo.mRCode == RCODE_BREAK) {
             fprintf(stderr, "invalid break. Not in a loop\n");
-            exit(1);
+            xyzsh_restore_signal_default();
+            return FALSE;
         }
         else if(runinfo.mRCode == RCODE_RETURN) {
             fprintf(stderr, "invalid return. Not in a function\n");
-            exit(1);
+            xyzsh_restore_signal_default();
+            return FALSE;
         }
         else if(runinfo.mRCode == RCODE_EXIT) {
         }
         else {
             fprintf(stderr, "run time error\n");
             fprintf(stderr, "%s", string_c_str(gErrMsg));
-            exit(1);
+
+            xyzsh_restore_signal_default();
+
+            return FALSE;
         }
     }
 
@@ -597,6 +531,8 @@ void xyzsh_load_file(char* fname, char** argv, int argc)
     }
 
     xyzsh_restore_signal_default();
+
+    return TRUE;
 }
 
 void xyzsh_opt_c(char* cmd, char** argv, int argc)
@@ -619,7 +555,7 @@ void xyzsh_opt_c(char* cmd, char** argv, int argc)
         for(i=0; i<argc; i++) {
             vector_add(argv2, STRING_NEW_GC(argv[i], FALSE));
         }
-        hash_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
+        uobject_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
 
         if(!run(block, gStdin, gStdout, &rcode, gRootObject, fun)) {
             if(rcode == RCODE_BREAK) {
@@ -635,8 +571,8 @@ void xyzsh_opt_c(char* cmd, char** argv, int argc)
                 fprintf(stderr, "%s", string_c_str(gErrMsg));
             }
         }
-        xyzsh_restore_signal_default();
         (void)vector_pop_back(gStackFrames);
+        xyzsh_restore_signal_default();
     }
     else {
         fprintf(stderr, "parser error\n");
@@ -651,7 +587,7 @@ void xyzsh_opt_c(char* cmd, char** argv, int argc)
 
 char* xyzsh_job_title(int n)
 {
-    if(n < vector_size(gJobs) && n >= 0) {
+    if(n < vector_count(gJobs) && n >= 0) {
         sObject* job = vector_item(gJobs, n);
         return SJOB(job).mName;
     }
@@ -662,7 +598,7 @@ char* xyzsh_job_title(int n)
 
 int xyzsh_job_num()
 {
-    return vector_size(gJobs);
+    return vector_count(gJobs);
 }
 
 void xyzsh_add_inner_command(sObject* object, char* name, fXyzshNativeFun command, int arg_num, ...)
@@ -679,6 +615,6 @@ void xyzsh_add_inner_command(sObject* object, char* name, fXyzshNativeFun comman
     }
     va_end(args);
 
-    hash_put(object, name, nfun);
+    uobject_put(object, name, nfun);
 }
 
